#pragma once
#include <chrono>
#include "gtkmm.h"

class delta_timer {
public:
    delta_timer(const sigc::slot<void(double)>& tick, const uint32_t interval) {
        tick_ = tick;
        timer_ = Glib::signal_timeout().connect(sigc::mem_fun(*this, &delta_timer::on_timer), interval);
        last_time_ = std::chrono::steady_clock::now();
    }
    ~delta_timer() {
        timer_.disconnect();
    }
    bool on_timer() {
        const auto now = std::chrono::steady_clock::now();
        const std::chrono::nanoseconds& delta_time_ns = std::chrono::duration_cast<std::chrono::nanoseconds>(now - last_time_);
        last_time_ = now;
        // ns to s
        tick_(static_cast<float>(delta_time_ns.count()) / 1e9);
        return true;
    }
private:
    sigc::connection timer_;
    sigc::slot<void(double)> tick_;
    std::chrono::time_point<std::chrono::steady_clock> last_time_;
};
