#include "w_audio_buffer_bar.h"

#include "arona_application.h"
#include "audio/device/audio_device_manager.h"
#include "tool/delta_timer.h"

w_audio_buffer_bar::w_audio_buffer_bar() {
    timer_ = std::make_shared<delta_timer>(mem_fun(*this, &w_audio_buffer_bar::on_timer), 1000.f / get_arona_application()->get_screen_refresh_rate());
}

void w_audio_buffer_bar::set_buffers(weak_buffer in_buffers) {
    buffers = in_buffers;
    ref_buffer ref_buf = *buffers.lock().get();

    for (auto& buffer : ref_buf) {
        buffer.PushZeros(buffer.GetCapacity());
    }
    const int32_t diff_widget_buffer = volume_bars_.size() - ref_buf.size();
    if (diff_widget_buffer > 0) {
        for (int32_t i = 0; i < diff_widget_buffer; i++) {
            volume_bars_.pop_back();
        }
    } else if (diff_widget_buffer < 0) {
        for (int32_t i = 0; i < -diff_widget_buffer; i++) {
            auto bar = volume_bars_.emplace_back(create_volume_bar());
            append(*bar);
        }
    }
}

w_volume_bar* w_audio_buffer_bar::create_volume_bar() {
    return Gtk::make_managed<w_volume_bar>(true);
}

void w_audio_buffer_bar::on_timer(double delta_time) {
    if (buffers.expired()) {
        return;
    }
    ref_buffer ref_buf = *buffers.lock().get();

    const uint32_t sample_rate = get_audio_device_manager()->get_sample_rate();
    const auto delta_samples = static_cast<uint32_t>(delta_time * sample_rate);
    static std::vector<float> temp;

    for (int i = 0; i < ref_buf.size(); ++i) {
        auto& bar = volume_bars_[i];
        circular_audio_buffer<float>& buffer = ref_buf[i];
        const uint32_t available_samples = std::min(buffer.Num(), delta_samples);
        if (available_samples == 0) {
            continue;
        }
        temp.resize(available_samples);

        buffer.Pop(temp.data(), available_samples);

        float min_sample = *std::ranges::max_element(temp);
        float max_sample = *std::ranges::min_element(temp);

        float peak = std::max(std::abs(min_sample), max_sample);

        bar->set_value(peak);
    }
}
